@@ -2,24 +2,15 @@
 #undef SYSCONFDIR
 #undef HELPDIR
 #undef PLUGINSDIR
-#undef LOCALEDIR
-
-/* gtk / gnome */
-#undef HAVE_GTK
-#undef HAVE_IMLIB
-#undef HAVE_GNOME
-#undef HAVE_GNOME_PANEL
-
-/* hebrew support for GtkText */
-#undef GTK_HEBREW
-#undef GTK_HEBREW_RC
 
 /* misc.. */
 #undef MEM_DEBUG
 #undef HAVE_IPV6
 #undef HAVE_POPT_H
 #undef HAVE_SOCKS_H
-#undef HAVE_PERL
+#undef HAVE_PL_PERL
+#undef HAVE_STATIC_PERL
+#undef HAVE_GMODULE
 
 /* macros/curses checks */
 #undef HAS_CURSES
@@ -29,12 +20,14 @@
 #undef USE_NCURSES
 #undef NO_COLOR_CURSES
 #undef SCO_FLAVOR
-#undef NCURSES_970530
 
 /* our own curses checks */
+#undef USE_CURSES_WINDOWS
 #undef HAVE_NCURSES_USE_DEFAULT_COLORS
 #undef HAVE_CURSES_IDCOK
 #undef HAVE_CURSES_RESIZETERM
+#undef HAVE_CURSES_WRESIZE
+#undef USE_CURSES_WINDOWS
 
 /* nls */
 #undef ENABLE_NLS
