@@ -31,6 +31,10 @@
 #include "themes.h"
 #include "translation.h"
 
+static const char *format_backs = "04261537";
+static const char *format_fores = "kbgcrmyw";
+static const char *format_boldfores = "KBGCRMYW";
+
 static int signal_gui_print_text;
 static int hide_text_style;
 
@@ -57,9 +61,6 @@ int format_find_tag(const char *module, const char *tag)
 
 int format_expand_styles(GString *out, char format)
 {
-	static const char *backs = "04261537";
-	static const char *fores = "kbgcrmyw";
-	static const char *boldfores = "KBGCRMYW";
 	char *p;
 
 	switch (format) {
@@ -92,7 +93,9 @@ int format_expand_styles(GString *out, char format)
 		g_string_append_c(out, FORMAT_STYLE_INDENT);
 		break;
 	case 'F':
-		/* flashing - ignore */
+		/* blink */
+		g_string_append_c(out, 4);
+		g_string_append_c(out, FORMAT_STYLE_BLINK);
 		break;
 	case 'n':
 	case 'N':
@@ -102,30 +105,30 @@ int format_expand_styles(GString *out, char format)
 		break;
 	default:
 		/* check if it's a background color */
-		p = strchr(backs, format);
+		p = strchr(format_backs, format);
 		if (p != NULL) {
 			g_string_append_c(out, 4);
 			g_string_append_c(out, FORMAT_COLOR_NOCHANGE);
-			g_string_append_c(out, (char) ((int) (p-backs)+'0'));
+			g_string_append_c(out, (char) ((int) (p-format_backs)+'0'));
 			break;
 		}
 
 		/* check if it's a foreground color */
 		if (format == 'p') format = 'm';
-		p = strchr(fores, format);
+		p = strchr(format_fores, format);
 		if (p != NULL) {
 			g_string_append_c(out, 4);
-			g_string_append_c(out, (char) ((int) (p-fores)+'0'));
+			g_string_append_c(out, (char) ((int) (p-format_fores)+'0'));
 			g_string_append_c(out, FORMAT_COLOR_NOCHANGE);
 			break;
 		}
 
 		/* check if it's a bold foreground color */
 		if (format == 'P') format = 'M';
-		p = strchr(boldfores, format);
+		p = strchr(format_boldfores, format);
 		if (p != NULL) {
 			g_string_append_c(out, 4);
-			g_string_append_c(out, (char) (8+(int) (p-boldfores)+'0'));
+			g_string_append_c(out, (char) (8+(int) (p-format_boldfores)+'0'));
 			g_string_append_c(out, FORMAT_COLOR_NOCHANGE);
 			break;
 		}
@@ -212,15 +215,17 @@ void format_create_dest(TEXT_DEST_REC *dest,
 		window_find_closest(server, target, level);
 
 	dest->hilight_priority = 0;
-        dest->hilight_color = 0;
-        dest->hilight_bg_color = 0;
+        dest->hilight_color = NULL;
 }
 
+/* Return length of text part in string (ie. without % codes) */
 int format_get_length(const char *str)
 {
         GString *tmp;
 	int len;
 
+        g_return_val_if_fail(str != NULL, 0);
+
         tmp = g_string_new(NULL);
 	len = 0;
 	while (*str != '\0') {
@@ -233,17 +238,87 @@ int format_get_length(const char *str)
 
 			/* %% or unknown %code, written as-is */
 			if (*str != '%')
-                                len++;
+				len++;
 		}
 
                 len++;
-                str++;
+		str++;
 	}
 
 	g_string_free(tmp, TRUE);
         return len;
 }
 
+/* Return how many characters in `str' must be skipped before `len'
+   characters of text is skipped. Like strip_real_length(), except this
+   handles %codes. */
+int format_real_length(const char *str, int len)
+{
+	GString *tmp;
+	const char *start;
+
+        g_return_val_if_fail(str != NULL, 0);
+        g_return_val_if_fail(len >= 0, 0);
+
+        start = str;
+        tmp = g_string_new(NULL);
+	while (*str != '\0' && len > 0) {
+		if (*str == '%' && str[1] != '\0') {
+			str++;
+			if (*str != '%' && format_expand_styles(tmp, *str)) {
+                                str++;
+				continue;
+			}
+
+			/* %% or unknown %code, written as-is */
+			if (*str != '%') {
+				if (--len == 0)
+                                        break;
+			}
+		}
+
+                len--;
+		str++;
+	}
+
+	g_string_free(tmp, TRUE);
+        return (int) (str-start);
+}
+
+char *format_string_expand(const char *text)
+{
+	GString *out;
+	char code, *ret;
+
+        g_return_val_if_fail(text != NULL, NULL);
+
+	out = g_string_new(NULL);
+
+	code = 0;
+	while (*text != '\0') {
+		if (code == '%') {
+			/* color code */
+			if (!format_expand_styles(out, *text)) {
+				g_string_append_c(out, '%');
+				g_string_append_c(out, '%');
+				g_string_append_c(out, *text);
+			}
+			code = 0;
+		} else {
+			if (*text == '%')
+				code = *text;
+			else
+				g_string_append_c(out, *text);
+		}
+
+		text++;
+	}
+
+	ret = out->str;
+	g_string_free(out, FALSE);
+	return ret;
+}
+
 static char *format_get_text_args(TEXT_DEST_REC *dest,
 				  const char *text, char **arglist)
 {
@@ -772,6 +847,9 @@ void format_send_to_gui(TEXT_DEST_REC *dest, const char *text)
 			/* user specific colors */
 			flags &= ~PRINTFLAG_MIRC_COLOR;
 			switch (*ptr) {
+			case FORMAT_STYLE_BLINK:
+				flags ^= PRINTFLAG_BLINK;
+				break;
 			case FORMAT_STYLE_UNDERLINE:
 				flags ^= PRINTFLAG_UNDERLINE;
 				break;
@@ -813,6 +891,7 @@ void format_send_to_gui(TEXT_DEST_REC *dest, const char *text)
 		case 15:
 			/* remove all styling */
 			fgcolor = bgcolor = -1;
+			flags &= PRINTFLAG_INDENT;
 			break;
 		case 22:
 			/* reverse */
